/*
** SPDX-License-Identifier: BSD-3-Clause
** Copyright Contributors to the OpenEXR Project.
*/

#ifndef OPENEXR_CORE_H
#define OPENEXR_CORE_H

#include "openexr_conf.h"

#include "openexr_base.h"
#include "openexr_errors.h"

#include "openexr_context.h"
#include "openexr_attr.h"

#include "openexr_part.h"

#include "openexr_std_attr.h"

#include "openexr_chunkio.h"

#include "openexr_decode.h"
#include "openexr_encode.h"

#include "openexr_debug.h"

#endif /* OPENEXR_CORE_H */
